"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InsightsByPeriodRepository = void 0;
const config_1 = require("@n8n/config");
const di_1 = require("@n8n/di");
const typeorm_1 = require("@n8n/typeorm");
const zod_1 = require("zod");
const sql_1 = require("../../../../utils/sql");
const insights_by_period_1 = require("../entities/insights-by-period");
const insights_shared_1 = require("../entities/insights-shared");
const dbType = di_1.Container.get(config_1.GlobalConfig).database.type;
const summaryParser = zod_1.z
    .object({
    period: zod_1.z.enum(['previous', 'current']),
    type: zod_1.z.union([zod_1.z.literal(0), zod_1.z.literal(1), zod_1.z.literal(2), zod_1.z.literal(3)]),
    total_value: zod_1.z.union([zod_1.z.number(), zod_1.z.string()]),
})
    .array();
let InsightsByPeriodRepository = class InsightsByPeriodRepository extends typeorm_1.Repository {
    constructor(dataSource) {
        super(insights_by_period_1.InsightsByPeriod, dataSource.manager);
    }
    escapeField(fieldName) {
        return this.manager.connection.driver.escape(fieldName);
    }
    getPeriodFilterExpr(maxAgeInDays = 0) {
        let periodStartExpr = `date('now', '-${maxAgeInDays} days')`;
        if (dbType === 'postgresdb') {
            periodStartExpr = `CURRENT_DATE - INTERVAL '${maxAgeInDays} day'`;
        }
        else if (dbType === 'mysqldb' || dbType === 'mariadb') {
            periodStartExpr = `DATE_SUB(CURRENT_DATE, INTERVAL ${maxAgeInDays} DAY)`;
        }
        return periodStartExpr;
    }
    getPeriodStartExpr(periodUnitToCompactInto) {
        let periodStartExpr = periodUnitToCompactInto === 'week'
            ? "strftime('%Y-%m-%d 00:00:00.000', date(periodStart, 'weekday 0', '-6 days'))"
            : `strftime('%Y-%m-%d ${periodUnitToCompactInto === 'hour' ? '%H' : '00'}:00:00.000', periodStart)`;
        if (dbType === 'mysqldb' || dbType === 'mariadb') {
            periodStartExpr =
                periodUnitToCompactInto === 'week'
                    ? "DATE_FORMAT(DATE_SUB(periodStart, INTERVAL WEEKDAY(periodStart) DAY), '%Y-%m-%d 00:00:00')"
                    : `DATE_FORMAT(periodStart, '%Y-%m-%d ${periodUnitToCompactInto === 'hour' ? '%H' : '00'}:00:00')`;
        }
        else if (dbType === 'postgresdb') {
            periodStartExpr = `DATE_TRUNC('${periodUnitToCompactInto}', ${this.escapeField('periodStart')})`;
        }
        return periodStartExpr;
    }
    getPeriodInsightsBatchQuery({ periodUnitToCompactFrom, compactionBatchSize, maxAgeInDays, }) {
        const batchQuery = this.createQueryBuilder()
            .select(['id', 'metaId', 'type', 'periodStart', 'value'].map((fieldName) => this.escapeField(fieldName)))
            .where(`${this.escapeField('periodUnit')} = ${insights_shared_1.PeriodUnitToNumber[periodUnitToCompactFrom]}`)
            .andWhere(`${this.escapeField('periodStart')} < ${this.getPeriodFilterExpr(maxAgeInDays)}`)
            .orderBy(this.escapeField('periodStart'), 'ASC')
            .limit(compactionBatchSize);
        return batchQuery;
    }
    getAggregationQuery(periodUnit) {
        const periodStartExpr = this.getPeriodStartExpr(periodUnit);
        const aggregationQuery = this.manager
            .createQueryBuilder()
            .select(this.escapeField('metaId'))
            .addSelect(this.escapeField('type'))
            .addSelect(insights_shared_1.PeriodUnitToNumber[periodUnit].toString(), 'periodUnit')
            .addSelect(periodStartExpr, 'periodStart')
            .addSelect(`SUM(${this.escapeField('value')})`, 'value')
            .from('rows_to_compact', 'rtc')
            .groupBy(this.escapeField('metaId'))
            .addGroupBy(this.escapeField('type'))
            .addGroupBy(periodStartExpr);
        return aggregationQuery;
    }
    async compactSourceDataIntoInsightPeriod({ sourceBatchQuery, sourceTableName = this.metadata.tableName, periodUnitToCompactInto, }) {
        const getBatchAndStoreInTemporaryTable = (0, sql_1.sql) `
			CREATE TEMPORARY TABLE rows_to_compact AS
			${sourceBatchQuery.getSql()};
		`;
        const countBatch = (0, sql_1.sql) `
			SELECT COUNT(*) ${this.escapeField('rowsInBatch')} FROM rows_to_compact;
		`;
        const targetColumnNamesStr = ['metaId', 'type', 'periodUnit', 'periodStart']
            .map((param) => this.escapeField(param))
            .join(', ');
        const targetColumnNamesWithValue = `${targetColumnNamesStr}, value`;
        const aggregationQuery = this.getAggregationQuery(periodUnitToCompactInto);
        const insertQueryBase = (0, sql_1.sql) `
			INSERT INTO ${this.metadata.tableName}
				(${targetColumnNamesWithValue})
			${aggregationQuery.getSql()}
		`;
        let deduplicateQuery;
        if (dbType === 'mysqldb' || dbType === 'mariadb') {
            deduplicateQuery = (0, sql_1.sql) `
				ON DUPLICATE KEY UPDATE value = value + VALUES(value)`;
        }
        else {
            deduplicateQuery = (0, sql_1.sql) `
				ON CONFLICT(${targetColumnNamesStr})
				DO UPDATE SET value = ${this.metadata.tableName}.value + excluded.value
				RETURNING *`;
        }
        const upsertEvents = (0, sql_1.sql) `
			${insertQueryBase}
			${deduplicateQuery}
		`;
        const deleteBatch = (0, sql_1.sql) `
			DELETE FROM ${sourceTableName}
			WHERE id IN (SELECT id FROM rows_to_compact);
		`;
        const dropTemporaryTable = (0, sql_1.sql) `
			DROP TABLE rows_to_compact;
		`;
        const result = await this.manager.transaction(async (trx) => {
            await trx.query(getBatchAndStoreInTemporaryTable);
            await trx.query(upsertEvents);
            const rowsInBatch = await trx.query(countBatch);
            await trx.query(deleteBatch);
            await trx.query(dropTemporaryTable);
            return Number(rowsInBatch[0].rowsInBatch);
        });
        return result;
    }
    async getPreviousAndCurrentPeriodTypeAggregates() {
        const cte = dbType === 'sqlite'
            ? (0, sql_1.sql) `
						SELECT
							datetime('now', '-7 days') AS current_start,
							datetime('now') AS current_end,
							datetime('now', '-14 days') AS previous_start
					`
            : dbType === 'postgresdb'
                ? (0, sql_1.sql) `
								SELECT
								(NOW() - INTERVAL '7 days')::timestamptz AS current_start,
								NOW()::timestamptz AS current_end,
								(NOW() - INTERVAL '14 days')::timestamptz AS previous_start
							`
                : (0, sql_1.sql) `
								SELECT
									DATE_SUB(NOW(), INTERVAL 7 DAY) AS current_start,
									NOW() AS current_end,
									DATE_SUB(NOW(), INTERVAL 14 DAY) AS previous_start
							`;
        const rawRows = await this.createQueryBuilder('insights')
            .addCommonTableExpression(cte, 'date_ranges')
            .select((0, sql_1.sql) `
						CASE
							WHEN insights.periodStart >= date_ranges.current_start AND insights.periodStart <= date_ranges.current_end
							THEN 'current'
							ELSE 'previous'
						END
					`, 'period')
            .addSelect('insights.type', 'type')
            .addSelect('SUM(value)', 'total_value')
            .innerJoin('date_ranges', 'date_ranges', '1=1')
            .where('insights.periodStart >= date_ranges.previous_start')
            .andWhere('insights.periodStart <= date_ranges.current_end')
            .groupBy('period')
            .addGroupBy('insights.type')
            .getRawMany();
        return summaryParser.parse(rawRows);
    }
};
exports.InsightsByPeriodRepository = InsightsByPeriodRepository;
exports.InsightsByPeriodRepository = InsightsByPeriodRepository = __decorate([
    (0, di_1.Service)(),
    __metadata("design:paramtypes", [typeorm_1.DataSource])
], InsightsByPeriodRepository);
//# sourceMappingURL=insights-by-period.repository.js.map